/***************************************************************************
 *   Copyright (C) 2006 by Michael Andreen                                 *
 *   andreen@student.chalmers.se                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA          *
 ***************************************************************************/

#ifndef __GENEALGORITHMS_H__
#define __GENEALGORITHMS_H__

#include <vector>

class GeneOrder;

struct Component{
	Component(int b,int e,int s):begin(b),end(e),sign(s){}
	bool operator==(const Component& c){
		return begin == c.begin && end == c.end && sign == c.sign;
	}
	int begin;
	int end;
	int sign;
};

struct Interval{
	Interval(size_t f,size_t s,bool o = false):first(f),second(s),oriented(o){}
	bool operator==(const Interval& i){
		return first == i.first && second == i.second && oriented == i.oriented;
	}
	size_t first;
	size_t second;
	bool oriented;
};

/**
 * Returns the length of the longest increasing sequence and the longest
 * decreasing sequence.
 */
std::pair<int,int> longestSequences(const GeneOrder& go);

/**
 */
std::vector<std::vector<int> > robinsonSchensted(const GeneOrder& go);

/**
 * Counts the number of cycles in the gene order.
 */
size_t countCycles(const GeneOrder& go);

/**
 * Calculates the inversion distance for this gene order
 */
size_t inversionDistance(const GeneOrder& go);

/**
 * Finds the components in the gene order.
 */
std::vector<Component> findComponents(const GeneOrder& go);

/**
 * Find intervals.
 */
std::vector<Interval> findIntervals(const GeneOrder& go);

/**
 * Creates a list with the intervals at each point.
 */
std::vector<Interval> findIntervalsAtPoints(const std::vector<Interval>& intervals);

#endif

