/***************************************************************************
 *   Copyright (C) 2006 by Michael Andreen                                 *
 *   andreen@student.chalmers.se                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA          *
 ***************************************************************************/

#ifndef __MODELIDENTIFIER_H__
#define __MODELIDENTIFIER_H__

#include "geneorder.h"

#include <queue>
#include <string>
#include <stdexcept>

struct fann;
class Model;


/**
 * Identifies the model this gene order belongs to
 * \author Michael Andreen
 */
class ModelIdentifier{
	public:

		/**
		 * Creates a new identifier given an artificial neural network
		 *
		 * \param ann filename, including path, to the neural network
		 *
		 * \throws std::invalid_argument exception if the ann could not be
		 * opened
		 */
		ModelIdentifier(std::string ann);

		/**
		 * Identifies the model for a given gene order.
		 *
		 * \returns a map with the model as key and the score between -1 and 1
		 */
		std::priority_queue<std::pair<double,Model> > identify(const GeneOrder& go);

		~ModelIdentifier();
	private:
		fann* _ann;

		//disable for the moment
		void operator=(const ModelIdentifier&){}

};

#endif

