/***************************************************************************
 *   Copyright (C) 2006 by Michael Andreen                                 *
 *   andreen@student.chalmers.se                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA          *
 ***************************************************************************/

#ifndef __SORTACTION_H__
#define __SORTACTION_H__

#include <tr1/memory>
#include <string>
class GeneOrder;


class SortActionImpl{
	public:

		virtual ~SortActionImpl(){};

		/**
		 * Applies the action on the GeneOrder and returning it.
		 */
		virtual GeneOrder& operator()(GeneOrder& go) const = 0;

		/**
		 * Compares sort actions.
		 */
		virtual bool operator==(const SortActionImpl& sa) const = 0;
		
		/**
		 * Gives a string representation of the action, for output
		 */
		virtual std::string toString() const = 0;
};

/**
 * Abstraction of a sort action, all child actions has to be immutable.
 *
 * \author Michael Andreen
 */
class SortAction{
	public:
		typedef std::tr1::shared_ptr<SortActionImpl> ActionPointer;

		/**
		 * Creates a new sort action, given a specified action.
		 * SortAction promises to remove the given action.
		 */
		SortAction(SortActionImpl* sa): _action(sa){
		}

		SortAction(const SortAction& sa): _action(sa._action){
		}

		const SortAction& operator=(const SortAction& sa){
			if (this != &sa)
				_action = sa._action;
			return *this;
		}
		
		~SortAction(){};

		/**
		 * Applies the action on the GeneOrder and returning it.
		 */
		GeneOrder& operator()(GeneOrder& go) const{
			return (*_action)(go);
		}

		/**
		 * Compares sort actions.
		 */
		bool operator==(const SortAction& sa) const{
			return (*_action) == (*sa._action);
		}

		/**
		 * Compares sort actions.
		 */
		bool operator==(const SortActionImpl& sa) const{
			return (*_action) == sa;
		}
		
		/**
		 * Gives a string representation of the action, for output
		 */
		std::string toString() const{
			return _action->toString();
		}

		/**
		 * Gives access to the implementation
		 */
		const SortActionImpl& impl(){
			return *_action;
		}
	private:
		ActionPointer _action;

};

#endif

